package com.ruoyi.sim.service.impl;

import com.ruoyi.sim.config.SimConfig;
import com.ruoyi.sim.domain.Sim;
import com.ruoyi.sim.domain.SimMsg;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Objects;

import static com.ruoyi.sim.service.impl.IotService.Const.*;

/**
 * 硬件通信
 * send service.
 */
@Service
public class IotService {

    interface Const {

        String ROUTER_IP = "127.0.0.1";
        String IP = "123.112.16.165";
        int PORT = 8899;
        /**
         * 报文长度
         */
        int MSG_LENGTH = 20;

        String PREFIX = "AA";

        String SUFFIX = "55";

        /**
         * orientation
         */
        String ORN_SEND = "01";

        /**
         * orientation
         */
        String ORN_RECEIVE = "02";

        String CMD_DATA_PLACE_HOLDER = "00000000";

        /**
         * 故障下发
         */
        String CMD_SET_FAULT = "01";
        /**
         * 故障清清除
         */
        String CMD_CLEAR_FAULT = "02";
        /**
         * 设备类型读取
         */
        String CMD_READ_TYPE = "03";
        /**
         * 状态读取 RESISTANCE电阻值
         */
        String CMD_READ_FAULT_RESISTANCE = "03";

        String CMD_ID_GET_SN = "B1";
        int LENGTH_2 = 2;
        int LENGTH_8 = 8;
        int LENGTH_20 = 20;
    }

    private static final Logger l = LoggerFactory.getLogger(IotService.class);
    /**
     * 缓存的Socket连接。
     */
    private Socket cachedSocket = null;
    @Autowired
    private SimReceiveService simReceiveService;
    @Autowired
    private SimConfig sc;

    /**
     * 初始化方法，项目启动后自动运行。
     */
    public void init() {
        //
        try {
            if (!isReachable(ROUTER_IP)) {
                // todo:ping 不通。
            }
            if (!isReachable(IP)) {
                // todo:ping 不通。
            }
            openSocket();
            //
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 定时。
     */
    public void scheduledCheckAllSimState() {

    }

    /**
     * 查找所有没有被手动禁用，并order by sim_num的模拟器列表。检查所有模拟器状态。
     */
    public void checkAllSimState() {

    }

    public void checkOneSimState(final Sim s) {
        if (Objects.isNull(s)) {
            return;
        }
        try {
            SimMsg sm = new SimMsg();
            // "s.getSimNum()"
            String sendMsg = buildSendMsgReadSimType("s.getSimNum()");
            sm.setSendMsg(sendMsg);
            String receiveMsg = send(sendMsg);
            sm.setReceiveMsg(receiveMsg);
            simReceiveService.checkOneSimState(sm);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 设备类型读取
     *
     * @param simNum sim.sim_num
     */
    public String buildSendMsgReadSimType(final String simNum) {
        return buildSendMsg(simNum, CMD_READ_TYPE, CMD_ID_GET_SN);
    }

    /**
     * 故障下发
     *
     * @param simNum          sim.sim_num
     * @param bindHardwareMsg fault.bind_hardware_msg
     */
    public String buildSendMsgWriteFault(final String simNum, final String bindHardwareMsg) {
        return buildSendMsg(simNum, CMD_SET_FAULT, bindHardwareMsg);
    }

    /**
     * 状态读取
     *
     * @param simNum          sim.sim_num
     * @param bindHardwareMsg fault.bind_hardware_msg
     */
    public String buildSendMsgReadFaultResistance(final String simNum, final String bindHardwareMsg) {
        return buildSendMsg(simNum, CMD_READ_FAULT_RESISTANCE, bindHardwareMsg);
    }

    /**
     * 故障清清除
     *
     * @param simNum          sim.sim_num
     * @param bindHardwareMsg fault.bind_hardware_msg
     */
    public String buildSendMsgClearFault(final String simNum, final String bindHardwareMsg) {
        return buildSendMsg(simNum, CMD_CLEAR_FAULT, bindHardwareMsg);
    }

    public String buildSendMsg(final String simNum, final String cmd, final String cmdId) {
        return buildSendMsg(simNum, cmd, cmdId, CMD_DATA_PLACE_HOLDER);
    }

    public String buildSendMsg(final String simNum, final String cmd, final String cmdId, final String data) {
        if (StringUtils.isEmpty(simNum) || StringUtils.isEmpty(cmd) || StringUtils.isEmpty(cmdId) || StringUtils.isEmpty(data)) {
            throw new IllegalArgumentException("buildSendMsg isEmpty");
        }
        if (simNum.length() != LENGTH_2 || cmd.length() != LENGTH_2 || cmdId.length() != LENGTH_2 || data.length() != LENGTH_8) {
            throw new IllegalArgumentException("buildSendMsg length error");
        }
        StringBuffer m = new StringBuffer();
        m.append(PREFIX);
        m.append(simNum);
        m.append(ORN_SEND);
        m.append(cmd);
        m.append(cmdId);
        m.append(data);
        m.append(SUFFIX);
        final String mFinal = m.toString();
        if (mFinal.length() != LENGTH_20) {
            throw new IllegalArgumentException("buildSendMsg length error");
        }
        return mFinal;
    }

    /**
     * send hex message
     *
     * @param sendMsg
     * @return
     */
    public String send(final String sendMsg) throws IOException {
        l.info("sendMsg = " + sendMsg);
        String receiveMsg = null;
        if (cachedSocket == null) {
            openSocket();
        }
        InputStream is = cachedSocket.getInputStream();
        OutputStream os = cachedSocket.getOutputStream();
        os.write(hexStrToByteArrs(sendMsg));
        byte[] buffer = new byte[1024];
        int length = is.read(buffer);
        StringBuffer sbHex = new StringBuffer();
        for (int i = 0; i < length; i++) {
            sbHex.append(String.format("%02X", buffer[i]));
        }
        receiveMsg = sbHex.toString();
        l.info("receiveMsg = " + receiveMsg);
        return receiveMsg;
    }

    public void openSocket() throws IOException {

        if (cachedSocket == null) {
            cachedSocket = new Socket(IP, PORT);
        }
    }

    /**
     * todo:异常精细处理
     *
     * @throws IOException
     */
    public void closeSocket() throws IOException {
        if (cachedSocket != null) {
            cachedSocket.close();
        }
        cachedSocket = null;
    }

    public byte[] hexStrToByteArrs(String hexString) {
//        if (StringUtils.isEmpty(hexString)) {
//            return null;
//        }
        hexString = hexString.replaceAll(" ", "");
        int len = hexString.length();
        int index = 0;

        byte[] bytes = new byte[len / 2];

        while (index < len) {
            String sub = hexString.substring(index, index + 2);
            bytes[index / 2] = (byte) Integer.parseInt(sub, 16);
            index += 2;
        }
        return bytes;
    }

    /**
     * ping
     *
     * @param ipV4
     * @return
     * @throws IOException
     */
    public boolean isReachable(String ipV4) throws IOException {
        InetAddress ia = InetAddress.getByName(ipV4);
        return ia.isReachable(sc.getGatewayReachableTimeout());
    }
}
