package com.ruoyi.sim.service.impl;

import java.util.List;

import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.sim.mapper.RealExamFaultMapper;
import com.ruoyi.sim.domain.RealExamFault;

/**
 * 考试故障关联Service业务层处理
 *
 * @author tom
 * @date 2024-12-15
 */
@Service
public class RealExamFaultService {
    @Autowired
    private RealExamFaultMapper realExamFaultMapper;

    /**
     * 查询考试故障关联
     *
     * @param refId 考试故障关联主键
     * @return 考试故障关联
     */
    public RealExamFault selectRealExamFaultByRefId(Long refId) {
        return realExamFaultMapper.selectRealExamFaultByRefId(refId);
    }

    /**
     * 查询考试故障关联列表
     *
     * @param realExamFault 考试故障关联
     * @return 考试故障关联
     */
    public List<RealExamFault> selectRealExamFaultList(RealExamFault realExamFault) {
        return realExamFaultMapper.selectRealExamFaultList(realExamFault);
    }

    /**
     * 新增考试故障关联
     *
     * @param realExamFault 考试故障关联
     * @return 结果
     */
    public int insertRealExamFault(RealExamFault realExamFault) {
        realExamFault.setCreateTime(DateUtils.getNowDate());
        return realExamFaultMapper.insertRealExamFault(realExamFault);
    }

    /**
     * 修改考试故障关联
     *
     * @param realExamFault 考试故障关联
     * @return 结果
     */
    public int updateRealExamFault(RealExamFault realExamFault) {
        realExamFault.setUpdateTime(DateUtils.getNowDate());
        return realExamFaultMapper.updateRealExamFault(realExamFault);
    }

    /**
     * 批量删除考试故障关联
     *
     * @param refIds 需要删除的考试故障关联主键
     * @return 结果
     */
    public int deleteRealExamFaultByRefIds(Long[] refIds) {
        return realExamFaultMapper.deleteRealExamFaultByRefIds(refIds);
    }

    /**
     * 删除考试故障关联信息
     *
     * @param refId 考试故障关联主键
     * @return 结果
     */
    public int deleteRealExamFaultByRefId(Long refId) {
        return realExamFaultMapper.deleteRealExamFaultByRefId(refId);
    }
}
